Morph subclass: #SpriteHandleMorph
	instanceVariableNames: 'type offset target reversed '
	classVariableNames: 'Current '
	poolDictionaries: ''
	category: 'Scratch-UI-Support'!
!SpriteHandleMorph commentStamp: '<historical>' prior: 0!
I am an interactive sprite handle, sort of like a halo button in Scratch. I am meant to be a singleton. Currently I can only resize or rotate a sprite.!


!SpriteHandleMorph methodsFor: 'initialization' stamp: 'jens 11/27/2008 18:46'!
initialize

	super initialize.
	reversed _ false.
	self
		color: Color transparent;
		addMorph: ImageMorph new.
! !


!SpriteHandleMorph methodsFor: 'event handling' stamp: 'jens 10/27/2008 13:38'!
handlesMouseDown: evt
	"Return true if this morph wants to receive mouseDown events" 

	^true! !

!SpriteHandleMorph methodsFor: 'event handling' stamp: 'jens 10/27/2008 14:02'!
handlesMouseOver: evt

	^ true
! !

!SpriteHandleMorph methodsFor: 'event handling' stamp: 'jens 12/4/2008 15:23'!
mouseDown: evt

	#resize = type
		ifTrue: [
			offset _ evt hand cursorPoint x - self left.
			^self ].
	
	offset _ self targetBounds center dist: self center.! !

!SpriteHandleMorph methodsFor: 'event handling' stamp: 'ee 3/2/2009 10:47'!
mouseEnter: evt

	(#resize = type)
		ifTrue: [self form: (ScratchFrameMorph skinAt: #spriteResizeOver)]
		ifFalse: [self form: (ScratchFrameMorph skinAt: #spriteRotateOver)].
! !

!SpriteHandleMorph methodsFor: 'event handling' stamp: 'ee 3/2/2009 10:45'!
mouseLeave: evt

	(#resize = type)
		ifTrue: [self form: (ScratchFrameMorph skinAt: #spriteResize)]
		ifFalse: [self form: (ScratchFrameMorph skinAt: #spriteRotate)].
! !

!SpriteHandleMorph methodsFor: 'event handling' stamp: 'jens 12/4/2008 16:06'!
mouseMove: evt

	| dist tc |
	offset ifNil: [^self].

	#resize = type 
		ifTrue: [
			dist _ evt hand cursorPoint x - offset - self left.
			reversed
				ifTrue: [((dist negative & self atMax) | 
					(dist negative not & self atMin)) ifTrue: [^self ]]
				ifFalse: [((dist negative & self atMin) | 
					(dist negative not & self atMax)) ifTrue: [^self ]].
			^self 
				left: self left + dist;
				resizeTarget ].

	tc _ self targetBounds center.
	self 
		center: (tc + (Point r: offset degrees: (evt cursorPoint - tc)
			 theta radiansToDegrees rounded));
		rotateTarget


! !

!SpriteHandleMorph methodsFor: 'event handling' stamp: 'jens 10/27/2008 10:17'!
mouseUp: evt

	offset _ nil! !


!SpriteHandleMorph methodsFor: 'accessing' stamp: 'jens 3/9/2009 22:37'!
target: aSprite
	"show me"

	| stage tb x |
	target _ aSprite.
	stage _ target ownerThatIsA: ScratchStageMorph.
	tb _ self targetBounds.

	"make sure the target sprite is onstage"
	stage notNil & stage fullContainsPoint: target center.
	(stage notNil and: [stage bounds containsPoint: target center]) ifFalse: [
		target referencePosition: 0@0.
		tb _ self targetBounds].
	self	openInWorld.

	#resize = type
		ifTrue: [
			target xpos > 0
				ifTrue: [
					reversed _ true.
					x _ tb left]
				ifFalse: [x _ tb right].
			^self center: x @ tb center y].

	(target xpos > 0 & (target heading > 0))
		ifTrue: [
			reversed _ true.
			self position: (tb center + (Point r: (tb width/2) degrees: target rotationDegrees + 180))]
		ifFalse: [self center: (tb center + (Point r: (tb width/2) degrees: target rotationDegrees))].
! !


!SpriteHandleMorph methodsFor: 'private' stamp: 'jens 10/27/2008 08:18'!
form: aForm

	(self submorphs at: 1) form: aForm.
	self extent: (self submorphs at: 1) extent.
! !

!SpriteHandleMorph methodsFor: 'private' stamp: 'jens 12/4/2008 14:11'!
targetBounds
	"private - answer the target's bounds as it displayed on screen, i.e. in q-size mode"

	| stage ratio home center |

	ratio _ 1.
	stage _ target ownerThatIsA: ScratchStageMorph.
	stage notNil & stage isQuarterSize ifTrue: [ratio _ 0.5 ].
	home_ 0 @ 0.
	stage ifNotNil: [home _ stage position ].

	center _ target center - home * ratio + home.
	^Rectangle center: center extent: target extent * ratio! !

!SpriteHandleMorph methodsFor: 'private' stamp: 'jens 10/27/2008 12:51'!
type: aSymbol
	"#resize / #rotate"

	type _ aSymbol.
! !


!SpriteHandleMorph methodsFor: 'sprite handling' stamp: 'jens 12/4/2008 15:29'!
resizeTarget

	| factor  tb|
	tb _ self targetBounds.
	reversed
		ifTrue: [ factor _ (self center x - tb center x) / (tb left - tb center x) ]
		ifFalse: [ factor _ (self center x - tb center x) / (tb right - tb center x) ].

	target multiplySizeBy: factor! !

!SpriteHandleMorph methodsFor: 'sprite handling' stamp: 'jens 7/16/2009 00:16'!
rotateTarget

	reversed
		ifTrue: [target heading: (self center - self targetBounds center) 
			theta radiansToDegrees rounded - 90]
		ifFalse: [target heading: (self center - self targetBounds center) 
			theta radiansToDegrees rounded + 90 ]! !


!SpriteHandleMorph methodsFor: 'testing' stamp: 'CWB 7/11/2012 16:44'!
atMax
	"answer true if the target sprite cannot be made any bigger"

	| maxExtent |
	maxExtent _ ScratchFrameMorph stageExtent asFloatPoint * 1.5.
	^(target extent x >= maxExtent x) or:
		[target extent y >= maxExtent y]! !

!SpriteHandleMorph methodsFor: 'testing' stamp: 'jens 12/4/2008 15:47'!
atMin
	"answer true if the target sprite cannot be made any smaller"

	| minExtent|
	minExtent _ target costumeForm extent min: 10@10.
	^(target extent x <= minExtent x) or: [target extent y <= minExtent y].
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SpriteHandleMorph class
	instanceVariableNames: ''!

!SpriteHandleMorph class methodsFor: 'accessing' stamp: 'jens 10/27/2008 11:30'!
vanish
	"delete my single instance"

	Current ifNotNil: [
		Current delete.
		Current _ nil ]! !


!SpriteHandleMorph class methodsFor: 'instance creation' stamp: 'ee 3/2/2009 10:44'!
resize: aSprite
	"create my single instance and set it to resize aSprite"	

	self vanish.
	Current _ self new 
		type: #resize;
		form: (ScratchFrameMorph skinAt: #spriteResize); 
		target: aSprite;
		setBalloonText: 'drag to resize' localized.
	^ Current
! !

!SpriteHandleMorph class methodsFor: 'instance creation' stamp: 'ee 3/2/2009 10:44'!
rotate: aSprite
	"create my single instance and set it to rotate aSprite"	

	self vanish.
	Current _ self new 
		type: #rotate;
		form: (ScratchFrameMorph skinAt: #spriteRotate); 
		target: aSprite;
		setBalloonText: 'drag to rotate' localized.
	^ Current
! !
