LeafNode subclass: #VariableNode
	instanceVariableNames: 'name '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Compiler'!
!VariableNode commentStamp: '<historical>' prior: 0!
I am a parse tree leaf representing a variable. Note that my name and key are different for pool variables: the key is the Object Reference.!


!VariableNode methodsFor: 'initialize-release'!
asStorableNode: encoder
	^ self! !

!VariableNode methodsFor: 'initialize-release'!
name: varName index: i type: type
	"Only used for initting instVar refs"
	name _ varName.
	self key: varName
		index: i
		type: type! !

!VariableNode methodsFor: 'initialize-release'!
name: string key: object code: byte
	"Only used for initting std variables, nil, true, false, self, etc."
	name _ string.
	key _ object.
	code _ byte! !

!VariableNode methodsFor: 'initialize-release'!
name: varName key: objRef index: i type: type
	"Only used for initting global (litInd) variables"
	name _ varName.
	self key: objRef
		index: i
		type: type! !


!VariableNode methodsFor: 'testing'!
assignmentCheck: encoder at: location

	(encoder cantStoreInto: name)
		ifTrue: [^ location]
		ifFalse: [^ -1]
! !

!VariableNode methodsFor: 'testing'!
canBeSpecialArgument
	"Can I be an argument of (e.g.) ifTrue:?"

	^code < LdNil! !

!VariableNode methodsFor: 'testing' stamp: 'tk 8/2/1999 18:40'!
isSelfPseudoVariable
	"Answer if this ParseNode represents the 'self' pseudo-variable."

	^ key = 'self'! !

!VariableNode methodsFor: 'testing'!
isVariableReference

	^true! !

!VariableNode methodsFor: 'testing' stamp: 'di 2/3/1999 09:41'!
type
	"This code attempts to reconstruct the type from its encoding in code.
		This allows one to test, for instance, (aNode type = LdInstType)."
	| type |
	code < 0 ifTrue: [^ code negated].
	code < 256 ifFalse: [^ code // 256].
	type _ CodeBases findFirst: [:one | code < one].
	type = 0
		ifTrue: [^ 5]
		ifFalse: [^ type - 1]! !


!VariableNode methodsFor: 'code generation'!
emitForReturn: stack on: strm

	(code >= LdSelf and: [code <= LdNil])
		ifTrue: 
			["short returns"
			strm nextPut: EndMethod - 4 + (code - LdSelf).
			stack push: 1 "doesnt seem right"]
		ifFalse: 
			[super emitForReturn: stack on: strm]! !

!VariableNode methodsFor: 'code generation'!
emitForValue: stack on: strm

	code < 256
		ifTrue: 
			[strm nextPut: (code = LdSuper ifTrue: [LdSelf] ifFalse: [code]).
			stack push: 1]
		ifFalse: 
			[self emitLong: LoadLong on: strm.
			stack push: 1]! !

!VariableNode methodsFor: 'code generation'!
emitStore: stack on: strm

	self emitLong: Store on: strm! !

!VariableNode methodsFor: 'code generation'!
emitStorePop: stack on: strm
	(code between: 0 and: 7)
		ifTrue: 
			[strm nextPut: ShortStoP + code "short stopop inst"]
		ifFalse:
			[(code between: 16 and: 23)
				ifTrue: [strm nextPut: ShortStoP + 8 + code - 16 "short stopop temp"]
				ifFalse: [(code >= 256 and: [code \\ 256 > 63 and: [code // 256 = 4]])
						ifTrue: [self emitLong: Store on: strm. strm nextPut: Pop]
						ifFalse: [self emitLong: StorePop on: strm]]].
	stack pop: 1! !

!VariableNode methodsFor: 'code generation' stamp: 'di 2/6/2000 10:52'!
fieldOffset  "Return temp or instVar offset for this variable"

	code < 256
		ifTrue: 
			[^ code \\ 16]
		ifFalse: 
			[^ code \\ 256]! !

!VariableNode methodsFor: 'code generation'!
sizeForReturn: encoder

	(code >= LdSelf and: [code <= LdNil])
		ifTrue: ["short returns" ^1].
	^super sizeForReturn: encoder! !

!VariableNode methodsFor: 'code generation'!
sizeForStore: encoder
	self reserve: encoder.
	code < 256 ifTrue: [^ 2].
	(code \\ 256) <= 63 ifTrue: [^ 2].
	^ 3! !

!VariableNode methodsFor: 'code generation'!
sizeForStorePop: encoder
	self reserve: encoder.
	(code < 24 and: [code noMask: 8]) ifTrue: [^ 1].
	code < 256 ifTrue: [^ 2].
	code \\ 256 <= 63 ifTrue: [^ 2].  "extended StorePop"
	code // 256 = 1 ifTrue: [^ 3].  "dbl extended StorePopInst"
	code // 256 = 4 ifTrue: [^ 4].  "dbl extended StoreLitVar , Pop"
	self halt.  "Shouldn't get here"! !


!VariableNode methodsFor: 'printing' stamp: 'sw 11/16/1999 16:36'!
printOn: aStream indent: level 
	aStream withAttributes: (Preferences syntaxAttributesFor: #variable)
		do: [aStream nextPutAll: name].
! !


!VariableNode methodsFor: 'C translation' stamp: 'jm 11/15/2003 04:50'!
asTranslatorNode

	name = 'true' ifTrue: [^ TConstantNode new setValue: true].
	name = 'false' ifTrue: [^ TConstantNode new setValue: false].
	^ TVariableNode new setName: name
! !


!VariableNode methodsFor: 'elements' stamp: 'jens 1/20/2009 00:11'!
asSyntaxElement

	^ObjectElementMorph new
		label: name! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

VariableNode class
	instanceVariableNames: ''!

!VariableNode class methodsFor: 'class initialization'!
initialize    "VariableNode initialize.  Decompiler initialize"
	| encoder |
	encoder _ Encoder new.
	StdVariables _ Dictionary new: 16.
	encoder
		fillDict: StdVariables
		with: VariableNode
		mapping: #('self' 'thisContext' 'super' 'nil' 'false' 'true' )
		to: (Array with: LdSelf with: LdThisContext with: LdSuper)
				, (Array with: LdNil with: LdFalse with: LdTrue).
	StdSelectors _ Dictionary new: 64.
	encoder
		fillDict: StdSelectors
		with: SelectorNode
		mapping: ((1 to: Smalltalk specialSelectorSize) collect: 
							[:i | Smalltalk specialSelectorAt: i])
		to: (SendPlus to: SendPlus + 31).
	StdLiterals _ LiteralDictionary new: 16.
	encoder
		fillDict: StdLiterals
		with: LiteralNode
		mapping: #(-1 0 1 2 )
		to: (LdMinus1 to: LdMinus1 + 3).
	encoder initScopeAndLiteralTables.

	NodeNil _ encoder encodeVariable: 'nil'.
	NodeTrue _ encoder encodeVariable: 'true'.
	NodeFalse _ encoder encodeVariable: 'false'.
	NodeSelf _ encoder encodeVariable: 'self'.
	NodeThisContext _ encoder encodeVariable: 'thisContext'.
	NodeSuper _ encoder encodeVariable: 'super'! !


VariableNode initialize!
